@@ -1,24 +1,21 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 2.7.12-4996.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+/* Bison interface for Yacc-like parsers in C
+   
+      Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,10 +26,20 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_SMI_PARSER_SMI_TAB_H_INCLUDED
+# define YY_SMI_PARSER_SMI_TAB_H_INCLUDED
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int smidebug;
+#endif
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -135,109 +142,14 @@
      WRITE_SYNTAX = 351
    };
 #endif
-/* Tokens.  */
-#define DOT_DOT 258
-#define COLON_COLON_EQUAL 259
-#define UPPERCASE_IDENTIFIER 260
-#define LOWERCASE_IDENTIFIER 261
-#define NUMBER 262
-#define NEGATIVENUMBER 263
-#define NUMBER64 264
-#define NEGATIVENUMBER64 265
-#define BIN_STRING 266
-#define HEX_STRING 267
-#define QUOTED_STRING 268
-#define ACCESS 269
-#define AGENT_CAPABILITIES 270
-#define APPLICATION 271
-#define AUGMENTS 272
-#define BEGIN_ 273
-#define BITS 274
-#define CHOICE 275
-#define CONTACT_INFO 276
-#define CREATION_REQUIRES 277
-#define COUNTER32 278
-#define COUNTER64 279
-#define DEFINITIONS 280
-#define DEFVAL 281
-#define DESCRIPTION 282
-#define DISPLAY_HINT 283
-#define END 284
-#define ENTERPRISE 285
-#define EXPORTS 286
-#define EXTENDS 287
-#define FROM 288
-#define GROUP 289
-#define GAUGE32 290
-#define IDENTIFIER 291
-#define IMPLICIT 292
-#define IMPLIED 293
-#define IMPORTS 294
-#define INCLUDES 295
-#define INDEX 296
-#define INSTALL_ERRORS 297
-#define INTEGER 298
-#define INTEGER32 299
-#define INTEGER64 300
-#define IPADDRESS 301
-#define LAST_UPDATED 302
-#define MACRO 303
-#define MANDATORY_GROUPS 304
-#define MAX_ACCESS 305
-#define MIN_ACCESS 306
-#define MODULE 307
-#define MODULE_COMPLIANCE 308
-#define MODULE_IDENTITY 309
-#define NOT_ACCESSIBLE 310
-#define NOTIFICATIONS 311
-#define NOTIFICATION_GROUP 312
-#define NOTIFICATION_TYPE 313
-#define OBJECT 314
-#define OBJECT_GROUP 315
-#define OBJECT_IDENTITY 316
-#define OBJECT_TYPE 317
-#define OBJECTS 318
-#define OCTET 319
-#define OF 320
-#define ORGANIZATION 321
-#define OPAQUE 322
-#define PIB_ACCESS 323
-#define PIB_DEFINITIONS 324
-#define PIB_INDEX 325
-#define PIB_MIN_ACCESS 326
-#define PIB_REFERENCES 327
-#define PIB_TAG 328
-#define POLICY_ACCESS 329
-#define PRODUCT_RELEASE 330
-#define REFERENCE 331
-#define REVISION 332
-#define SEQUENCE 333
-#define SIZE 334
-#define STATUS 335
-#define STRING 336
-#define SUBJECT_CATEGORIES 337
-#define SUPPORTS 338
-#define SYNTAX 339
-#define TEXTUAL_CONVENTION 340
-#define TIMETICKS 341
-#define TRAP_TYPE 342
-#define UNIQUENESS 343
-#define UNITS 344
-#define UNIVERSAL 345
-#define UNSIGNED32 346
-#define UNSIGNED64 347
-#define VALUE 348
-#define VARIABLES 349
-#define VARIATION 350
-#define WRITE_SYNTAX 351
-
-
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 1288 "parser-smi.y"
 {
+/* Line 2053 of yacc.c  */
+#line 1297 "parser-smi.y"
+
     char           *text;	  		/* scanned quoted text       */
     char           *id;				/* identifier name           */
     int            err;				/* actually just a dummy     */
@@ -258,14 +170,29 @@ typedef union YYSTYPE
     struct Index   index;
     Module	   *modulePtr;
     SubjectCategories *subjectCategoriesPtr;
-}
-/* Line 1489 of yacc.c.  */
-#line 264 "parser-smi.tab.h"
-	YYSTYPE;
+
+
+/* Line 2053 of yacc.c  */
+#line 177 "parser-smi.tab.h"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int smiparse (void *YYPARSE_PARAM);
+#else
+int smiparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int smiparse (struct Parser *parserPtr);
+#else
+int smiparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
+#endif /* !YY_SMI_PARSER_SMI_TAB_H_INCLUDED  */
