@@ -9,7 +9,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: dump-xml.c 6635 2007-03-13 21:53:21Z schoenw $
+ * @(#) $Id: dump-xml.c 1772 2012-04-01 12:15:23Z schoenw $
  */
 
 /*
@@ -272,6 +272,8 @@ static char *getValueString(SmiValue *valuePtr, SmiType *typePtr)
 	break;
     case SMI_BASETYPE_UNKNOWN:
 	break;
+    case SMI_BASETYPE_POINTER:
+	break;
     case SMI_BASETYPE_OBJECTIDENTIFIER:
 	for (i = 0; i < valuePtr->len; i++) {
 	    sprintf(&s[strlen(s)], i ? ".%u" : "%u", valuePtr->value.oid[i]);
@@ -1018,7 +1020,8 @@ void initXml()
 	"xml",
 	dumpXml,
 	0,
-	SMIDUMP_DRIVER_CANT_UNITE,
+	SMIDUMP_DRIVER_CANT_UNITE
+	| SMIDUMP_DRIVER_CANT_YANG,
 	"intermediate SMI XML exchange format",
 	opt,
 	NULL
